# frozen_string_literal: true

module Hermes
  module Tools
    # Tools for specs
    module Specs
      def get_typed_value(type, iteration = 0)
        return type[iteration] if type.kind_of?(Array)
        type_array = case type
                     when 'String'
                       %w[alpha bravo charlie]
                     when 'Boolean'
                       [false, true, true]
                     when 'Hash'
                       [
                           {'alpha' => 'bravo'},
                           {'charlie' => 'delta'},
                           {'echo' => 'foxtrott'}
                       ]
                     when '[String]'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array[Coord]'
                       [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]
                     when 'Date'
                       [
                           [2002, 2, 3, 5],
                           [2003, 2, 3, 8],
                           [2004, 2, 3, 6]
                       ].map {|arr| Date.new(*arr)}
                     when 'DateTime'
                       [
                           [2001, 2, 3, 4, 4, 4],
                           [2002, 2, 3, 4, 4, 6],
                           [2003, 2, 3, 4, 4, 6]
                       ].map {|arr| DateTime.new(*arr)}
                     when 'Integer'
                       [1, 2, 3]
                     when 'Float'
                       [1.1, 2.2, 3.3]
                     when 'Siren'
                      ['123456780', '223456789', '323456789']
                     when 'Siret'
                      ['12345678901234', '22345678901234', '32345678901234']
                     else
                       puts "ERROR: Unknow type #{type}"
                       [nil, nil, nil]
                     end
        type_array[iteration]
      end

      def nice_type(type)
        case type
        when 'String'
          nice_type = :string
        when 'Boolean'
          nice_type = :boolean
        when 'Hash'
          nice_type = :object
        when 'Array'
          nice_type = :array
        when 'Date'
          nice_type = :date
        when 'DateTime'
          nice_type = :time
        when 'Integer'
          nice_type = :integer
        when 'Float'
          nice_type = :float
        else
          puts 'ERROR: Unknow type'
        end
        nice_type
      end

      def nice_typev2(type, opt={})
        # see https://swagger.io/docs/specification/data-models/data-types for more options
        case type
        when 'String'
          return { type: :string, **opt }
        when 'Boolean'
          return { type: :boolean, **opt }
        when 'Hash'
          return { type: :object, **opt }
        when 'Array'
          return { type: :array, **opt }
        when 'Date'
          return { type: :string, format: :date, **opt }
        when 'DateTime'
          return { type: :string, format: :datetime, **opt }
        when 'Integer'
          return { type: :integer, **opt }
        when 'Float'
          return { type: :float, **opt }
        else
          puts 'ERROR: Unknow type'
        end
      end

      def jsonify_params(params, opt={})
        schema = {
            type: :object,
            properties: {
                data: {
                    type: :object,
                    properties: {
                        attributes: {
                            type: :object,
                            properties: {},
                            required: opt.fetch(:required, [])
                        }
                    }
                    #required: []
                }
            }
            #required: []
        }
        params.each do |key, val|
          if val[:type] == :array
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: :array, items: { type: :string } }
          else
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: val[:type]}
          end
        end
        schema
      end

      def create_mail_tempate_default
        MailTemplate.create(default: true, body: '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
          <html style="margin:0 auto !important;padding:0 !important;height:100% !important;width:100% !important;">
            <head>
              <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
              <meta name="viewport" content="width=device-width">
              <meta http-equiv="X-UA-Compatible" content="IE=edge">
              <meta name="x-apple-disable-message-reformatting">
              <title>{{title}}</title>
              <!-- CSS Reset : BEGIN -->
              <!-- CSS Reset : END -->
              <!-- Progressive Enhancements : BEGIN -->
              <!-- Progressive Enhancements : END -->
              <style>
                a:link{color:#FFFFFF;text-decoration:none}
                a:visited{text-decoration:none;color:#FFFFFF}
                a:hover{text-decoration:none;color:#FFFFFF}
                a:active{text-decoration:none;color:#FFFFFF}
              </style>
              <style type="text/css">
                            *[x-apple-data-detectors],  /* iOS */
                            .x-gmail-data-detectors,    /* Gmail */
                            .x-gmail-data-detectors *,
                            .aBn {
                                border-bottom: 0 !important;
                                cursor: default !important;
                                color: inherit !important;
                                text-decoration: none !important;
                                font-size: inherit !important;
                                font-family: inherit !important;
                                font-weight: inherit !important;
                                line-height: inherit !important;
                            }
    
                            @media only screen and (min-device-width: 375px) and (max-device-width: 413px) { /* iPhone 6 and 6+ */
                                .email-container {
                                    min-width: 100% !important;
                                }
                            }
                            body{
                              font-family: Arial; 
                              font-size: 14px; 
                              color: #333333; 
                              line-height: 1.4;
                            }
          
                            table td {
                                  border-collapse: collapse;
                              
                              }
          
                              table th {
                                  border-collapse: collapse;
                              }
          
                              table {
                                  border-collapse: collapse;
                              }
          
          
                        /* Media Queries */
                        @media screen and (max-width: 600px) {
                            .email-container {
                                width: 100% !important;
                                margin: auto !important;
                            }
                            .fluid {
                                width: 100% !important;
                                max-width: 100% !important;
                                height: auto !important;
                                margin-left: auto !important;
                                margin-right: auto !important;
                            }
          
                            .fluid.logo {
                                max-width: 200px !important;
                            }
          
                            /* What it does: Forces table cells into full-width rows. */
                            .stack-column,
                            .stack-column-center {
                                display: block !important;
                                width: 100% !important;
                                max-width: 100% !important;
                                direction: ltr !important;
                                padding: 0 !important;
                            }
                            /* And center justify these ones. */
                            .stack-column-center {
                                text-align: center !important;
                            }
                            /* What it does: Generic utility class for centering. Useful for images, buttons, and nested tables. */
                            .center-on-narrow {
                                text-align: center !important;
                                display: block !important;
                                margin-left: auto !important;
                                margin-right: auto !important;
                                float: none !important;
                            }
                            table.center-on-narrow {
                                display: inline-block !important;
                            }
                            .header-table{
                                font-size: 16px;
                            }
                            
                            .show-on-mobile{
                              display: inline!important;
                            }
          
                            .hide-on-mobile{
                                display: none!important;
                            }
          
                            .mobile-no-padding{
                                padding: 0 !important;
                            }
                            .mobile-margin-auto{
                                margin-left: auto !important;
                                margin-right: auto !important;
                            }
                            
                            .stack-column > table{
                              min-height: 0!important;
                              height: auto!important;
                            }
          
                            .stack-column.ol-item{
                                padding-top: 15px !important;
                            }
          
                            .stack-column.ol-body{
                              padding: 10px 0 !important;
                              width: 90% !important;
                              max-width: 90% !important;
                              margin: auto;
                            }
                        }
              </style>
            </head>
            <body style="margin:0 auto !important;padding:0 6px !important;height:100% !important; margin:0; background-color:#f7f7f7;">
              <table align="center" border="0" cellpadding="0" cellspacing="0" width="600" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; margin: auto; max-width: 600px!important; width: 600px; background-color: white;" class="email-container" role="presentation">
                <tbody>
                  <tr>
                    <td style="text-align:center;font-family: Arial;font-size: 14px;color: #333333;padding: 8px 0;line-height: 1.4;font-style: italic;">{{title}}</td>
                  </tr>
                  <tr>  
                  <td>
                    <!-- Logo -->
                    <table role="presentation" cellspacing="0" cellpadding="0" border="0" align="center" width="100%" style="border-spacing:0 !important;border-collapse:collapse !important;table-layout:fixed !important;max-width: 600px;" class="email-container">
                      <tr>
                        <td style="padding: 8px;">
                            <a href="https://www.actionlogement.fr" target="_blank">
                              <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/f953d58026b3288f6bac326f3fdafb21/6141affbb93ee3484e3a0795/logo_al.png" height="40" class="fluid logo" alt="Action Logement">
                            </a>
                        </td>
                        <td style="padding: 8px;">
                          <a href="https://als-hermes-scoring-rec.be-idys.com" target="_blank" style="float: right" >
                            <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/9adf36a551889157a2bf6340e8238523/6141aedeb93ee3484e3a0776/logo_alin_copy.png"  height="40" class="fluid logo" alt="Action Logement" alt="Alin">
                          </a>
                        </td>
          
                      </tr>
                    </table>
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="background-color: #abe8e6; color: #ffffff; font-size: 26px; font-family: Arial; font-weight: bold; text-transform: uppercase; text-align: center; line-height: 1.5; border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; max-width: 600px;" class="email-container header-table" role="presentation">
                    <tbody>
                    <tr>
                      <td>
                        <img border="0" height="190" width="600" style="-ms-interpolation-mode: bicubic; display: block; white-space: pre;" class="fluid" src="https://als-hermes-scoring-rec.be-idys.com/sassets/48693a1e251e5fb06ff8389f97ce9736/6141b0007a6d0043058d780c/asset-1631694848-h0o8c2rh.JPG" alt="Alin Salaries">
                      </td>
                    </tr>
                    </tbody>
                    </table>
                  </td>
                </tr>
                
                </tbody>
              </table>
              <table align="center" border="0" cellpadding="0" cellspacing="0" width="600" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; margin: auto; max-width: 600px!important; width: 600px; background-color: white;" class="email-container" role="presentation">
                <tbody>
                <tr>
                  <td>
                    <!-- Contenu liste numérotée -->
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; max-width: 600px; margin: 0 15px; width: auto;  background-color: white;" class="email-container" role="presentation">
                    <tbody>
                      {{content}} 
                    </tbody>
                    </table>
                      <!-- Footer AL -->
                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="border-spacing: 0 !important; border-collapse: separate !important; table-layout: fixed !important; max-width: 600px; padding-top: 10px; background-color: white;" class="footer-rs email-container" role="presentation">
                      <tbody>
                      <tr>
                        <td align="center" style="padding-top: 64px">
                          <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/92ee6d1d29d4446c7320b5fdccee21de/617a9023b93ee34ab41a3106/separation_transparent.png" alt="separation">
                        </td>
                      </tr>
                      <tr>
                        <td style="font-weight: 800;color: #003f7a; text-align: center; padding: 32px">
                          <span style="font-size: large;">AYEZ LE REFLEXE ACTION LOGEMENT !</span><br/>
                          <a style="color: #003f7a !important;" href="https://www.actionlogement.fr" target="_blank">www.actionlogement.fr</a>
                        </td>
                      </tr>
                      <tr>
                          <td style="font-size: 12px; text-align: center; padding: 32px; color: white; background: #e41d4b;">
                          <p>
                            {{subject}}
                            Les mails envoyés à cette adresse ne sont pas traités, si vous avez une autre question merci de vous rendre sur votre espace en ligne.
                          </p>
                          <p>&nbsp;</p>
                          <p>
                            Action Logement Services - SAS au capital de 20.000.000 d euros <br>
                            Siège social : 19/21 quai d Austerlitz 75013 Paris - 824 541 148 RCS Paris <br>
                            Immatriculée à l ORIAS sous le numéro 17006232 - Société de financement agréée et contrôlée par l ACPR
                          </p>
                        </td>
                      </tbody>
                    </table>
                  </td>
                </tr>
                </tbody>
              </table>
            </body>
          </html>')
      end 

      def check_mutable_params(opts)
        model = opts[:model] 
        mutate_params = opts[:mutate_params]
        factory = opts[:factory]
        check_post = opts.has_key?(:check_post) ? opts[:check_post] : true
        check_put = opts.has_key?(:check_put) ? opts[:check_put] : true
        min_params = opts.has_key?(:min_params) ? opts[:min_params] : {}
        embedded_params = opts.has_key?(:embedded_params) ? opts[:embedded_params] : {}
        other_factory = opts.has_key?(:other_factory) ? opts[:other_factory] : []
        user_roles =  opts.has_key?(:user_roles) ? opts[:user_roles] : []
        auth_roles =  opts.has_key?(:auth_roles) ? opts[:auth_roles] : ['superadministrateur']
        not_checked_keys = opts.has_key?(:not_checked_keys) ? opts[:not_checked_keys] : []

        not_mutable_params = model.attribute_names.map(&:to_sym) - mutate_params.keys - not_checked_keys
        show_json = false

        describe "#{model.class.name} check mutable" do 
          let!(:responsible) {create(:user, auth_roles: auth_roles, roles: user_roles)}

          other_factory.each do |o|
            let!(:"#{o[:id]}") {create(o[:factory], (o[:params] || {}).merge(id: o[:id]))}
          end 

          if check_post
            describe 'post' do 
              mutate_params.each do |param, type|
                describe "with #{param.to_s} (#{type.to_s}) is valid" do
                  let(:params) { {"#{param}": get_typed_value(type)} }
                  it 'returns status code 201 and the value appears in json' do
                    authenticated_header(request, responsible)
                    post 'create', {data: { attributes: min_params.merge(params) }}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(201)
                    if (type == 'Date' || type == 'DateTime' || type.kind_of?(Array))
                      expect(json['data']['attributes'][param.to_s]).not_to eq('')
                    else
                      expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
                    end
                  end
                end
              end
  
              # not_mutable_params.each do |param|
              #   describe "with #{param.to_s} is not mutable"  do
              #     it 'returns status 422 and show error' do
              #       authenticated_header(request, responsible)
              #       post 'create', { data: { attributes: min_params.merge({"#{param}": "test"})}}.merge(embedded_params)
              #       ap json if show_json
              #       expect(response).to have_http_status(422)
              #       expect(json["errors"].first.symbolize_keys).to eq({"#{param}": "not mutable"})
              #     end
              #   end
              # end 
            end
          end 

          if check_put 
            describe 'put' do 
              let!(:default_model) {create(factory, min_params)}

              mutate_params.each do |param, type|
                describe "with #{param.to_s} (#{type.to_s}) is valid" do
                  let(:params) { {"#{param}": get_typed_value(type, 1)} }
                  it 'returns status code 200 and the value appears in json' do
                    authenticated_header(request, responsible)
                    put 'update', {id: default_model.id, data: { attributes: params }}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(200)
                    if (type == 'Date' || type == 'DateTime' || type.kind_of?(Array))
                      expect(json['data']['attributes'][param.to_s]).not_to eq('')
                    else
                      expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 1))
                    end
                  end
                end
              end
    
              # not_mutable_params.each do |param|
              #   describe "with #{param.to_s} is not mutable"  do
              #     it 'returns status 422 and show error' do
              #       authenticated_header(request, responsible)
              #       put 'update', { id: default_model.id.to_s, data: { attributes: {"#{param}": "test"}}}.merge(embedded_params)
              #       ap json if show_json
              #       expect(response).to have_http_status(422)
              #       expect(json["errors"].first.symbolize_keys).to eq({"#{param}": "not mutable"})
              #     end
              #   end
              # end 
            end 
          end 
        end 
      end 


      def execute_last_job(job)
        last_job = enqueued_jobs.select{|a| a[:job] == job }.last
        last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        last_job
      end 

    end
  end
end

include Hermes::Tools::Specs
