require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  ACCEPTABLE_PARAMS = %w[
    name
    code
    lessor_id
  ].freeze

  describe Api::Dmo::SignupUserNotificationPreferencesController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:user) { create(:user) }
    

    describe 'POST /signup_user_notification_preferences' do
      let(:email) {'test@email.com'}
      describe 'Create without user' do
        it 'returns status code 201' do
          post 'create', {
              data: {
                  attributes: {
                      email: email,
                      notification_key: 'user.new_reserved_offer', 
                      want_to_be_subscribed: false
                  }
              }
          }
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['email']).to eq(email)
        end
        it 'throw error when missign params' do 
          post 'create', {
              data: {
                  attributes: {
                      email: email,
                      notification_key: 'user.new_reserved_offer', 
                  }
              }
          }
          expect(response).to have_http_status(422)
        end 
      end
    end
    describe 'POST /create_multiple_preferences' do
      let(:email) {'test@email.com'}
      describe 'Create without user' do
        it 'returns status code 201' do
          post 'create_multiple_preferences', {
              data: {
                  attributes: {
                      email: email,
                      notifs:[
                        { notification_key: 'user.new_reserved_offer', 
                          want_to_be_subscribed: false},
                          {
                            notification_key: 'user.new_published_offer', 
                            want_to_be_subscribed: false
                          }
                      ]
                     
                  }
              }
          }
          expect(response).to have_http_status(201)
          expect(json['email']).to eq(email)
          expect(json['signup_user_notification_preference_ids'].length).to eq(2)
        end
        it 'throw error when missign params' do 
          post 'create_multiple_preferences', {
              data: {
                  attributes: {
                      email: email,
                      notifs:[
                        { notification_key: 'user.new_reserved_offer', 
                          want_to_be_subscribed: false},
                          {
                            notification_key: 'user.new_published_offer', 
                          }
                      ]
                  }
              }
          }
          expect(response).to have_http_status(422)
        end 
      end
    end

    describe 'GET /signup_user_notification_preferences/user_notifications_list' do
      it 'returns list of user notifications' do
        get 'user_notifications_list'
        expect(response).to have_http_status(200)
        expect(json.length).not_to eq(0)
        expect(json).not_to eq( NOTIFICATION_KEYS[:user].to_json)
      end
    end

  end
end
