require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  ACCEPTABLE_PARAMS = %w[
    name
    object_id
    reference_model_name
    email
    web
    slack
    sms
  ].freeze

  MUTATE_PARAMS = {

  }

  describe Api::Dmo::UpdateSubscriptionsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:superadministrateur) { create(:user, auth_roles: ['superadministrateur']) }
    let!(:user) { create(:user) }

    before do 
      ['key.num1', 'key.num2', 'key.num3'].each do |k|
        superadministrateur.subscription.subscribe_to_email(k)
        user.subscription.subscribe_to_email(k)
      end 
    end 

    describe 'GET /update_subscriptions' do
      describe 'as superadministrateur' do 
        it ' works with with filtering' do
          authenticated_header(request, superadministrateur)
          get 'index', {name: { '$in': [ 'key.num1'], object_id: { '$in': [ superadministrateur.id.to_s, user.id.to_s]}}}
          expect(response).to have_http_status(200)
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end

      describe 'as user' do 
        it 'returns status code 200' do
          authenticated_header(request, user)
          get 'index'
          expect(response).to have_http_status(200)
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(4)
        end
      end
    end
  end
end
