require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'InterfaceNuu_v0300.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::AddRequestTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }
  let!(:user1) { create(:user) }
  let!(:applicant1) {
    create(
        :applicant,
        user: user1,
        monthly_charges: {},
        work_contract_type: "APP",
        has_other_employers: true,
        nationality: "1",
        family_situation:'V',
        year_minus_one: Date.current.year,
        amount_year_minus_one: "123",
        birth_date:Date.current,
        civility:"1",
        workplace_insee_code: "",
        sne_profile_creation_date: Date.current - 20.days,
        foreign_workplace_postal_code: "754114",
        foreign_workplace_locality: "Villedechine",
        foreign_workplace_country: "Chine",
        )
  }

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  context 'with simple housing_request' do
    let!(:applicant) {create(:applicant)}
    let!(:hr) do
      create(
          :housing_request,
          als_id: "0000001",
          is_active: true,
          main_applicant: applicant1,
          id_counter: "alpha",

          internal_ref: "alpha",
          seeked_localisation_list: [
              {
                  "nom": "Fréjus",
                  "code": "83061",
                  "codeDepartement": "83",
                  "codeRegion": "93",
                  "codesPostaux": [
                      "83600",
                      "83370"
                  ],
                  "population": 53168,
                  "_score": 1
              },
              {
                  "nom": "Fréjzus",
                  "code": "84061",
                  "codeDepartement": "81",
                  "codeRegion": "92",
                  "codesPostaux": [
                      "80600",
                      "80370"
                  ],
                  "population": 53168,
                  "_score": 1
              },
          ],
          forcing_duplicate: true,
          electronical_renewal: true,
          seeked_housing_categories: ["J"],
          seeking_first_floor: true,
          seeking_elevator: true,
          seeked_max_rent: 451,

          widen_request: true,
          seeked_types: ["T1", "T2"],
          typology_range: ["T1"],
          housing_type_list:["HLM"],
          actual_housing_details: {
            "postal_code"=>"75004",
            "appartment"=>"12",
            "floor"=>"12",
            "stair"=>"12",
            "building"=>"12",
            "company_adress"=>"douzeee",
            "number"=>"12",
            "way"=>"douzeee",
            "known_place"=>"douzeee",
            "district"=>"Paris",
            "address_insee_code"=>"75104"}, 
                      postal_details: {"appartment_building"=>"12",
            "appartment_staircase"=>"12",
            "appartment_floor"=>"12",
            "appartment_number"=>"12",
            "address_number"=>"12",
            "address_street"=>"douze",
            "address_add_address"=>"douze",
            "address_place"=>"douze",
            "postal_code"=>"75002",
            "district"=>"Paris",
            "address_insee_code"=>"75102"},
                      owned_housing_details: {"type"=>nil,
            "postal_code"=>nil,
            "district"=>nil
          },
          requester_lastname: "alpha",
          requester_firstname: "alpha",
          is_house_owner: true,
          motivations: ["02","01"],
          sne_created_at: DateTime.current,
      )
    end
    let!(:xml1) do
      create(
          :xml,
          type: 'get_demande',
          reference_model_name: 'Hermes::HousingRequest',
          object_id: hr.id,
          xml: '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><ns1:interfaceNuu xmlns:ns1="http://nuu.application.i2/" versionInterface="04.00" versionFormulaire="04.00"><ns1:entete><ns1:typeFichier>DIS</ns1:typeFichier><ns1:dateFichier>2023-02-27T15:14:05</ns1:dateFichier></ns1:entete><ns1:demande><ns1:demandeLogement><ns1:numGuichet>G2175</ns1:numGuichet><ns1:numUnique>1110223024120G2175</ns1:numUnique><ns1:refInterne>000021424</ns1:refInterne><ns1:dateCreationDemande>2023-02-27</ns1:dateCreationDemande><ns1:forcerDoublon>false</ns1:forcerDoublon><ns1:renouvellementElec>true</ns1:renouvellementElec><ns1:precision></ns1:precision><ns1:ressourcesLoiEC>30000</ns1:ressourcesLoiEC><ns1:anru>false</ns1:anru><ns1:ressourcesMensuelles>true</ns1:ressourcesMensuelles><ns1:informationTraitement/><ns1:logementRecherche><ns1:categorieLogementRecherche code="I"/><ns1:refusRezDeChaussee>true</ns1:refusRezDeChaussee><ns1:refusSansAscenseur>true</ns1:refusSansAscenseur><ns1:demandeElargie>true</ns1:demandeElargie><ns1:infoProprietaire>false</ns1:infoProprietaire><ns1:listeNombrePieces><ns1:nombrePiecesRecherchees code="P2"/><ns1:nombrePiecesRecherchees code="P3"/></ns1:listeNombrePieces><ns1:listeLocalisationSouhaite><ns1:localisationSouhaite><ns1:commune code="77122"/><ns1:codePostal>77380</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="75118"/><ns1:codePostal>75018</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="94043"/><ns1:codePostal>94270</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="94080"/><ns1:codePostal>94300</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="75119"/><ns1:codePostal>75019</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="91086"/><ns1:codePostal>91070</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="78372"/><ns1:codePostal>78160</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="94042"/><ns1:codePostal>94340</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="94017"/><ns1:codePostal>94500</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="93046"/><ns1:codePostal>93190</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="95127"/><ns1:codePostal>95000</ns1:codePostal><ns1:seuilEPCIPremierQuartile>10674</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>Quartile 4</ns1:quartileAppartenance></ns1:localisationSouhaite></ns1:listeLocalisationSouhaite></ns1:logementRecherche><ns1:personnePhysique><ns1:demandeur><ns1:civilite code="2"/><ns1:nomPatronymique>Paris-Leclerc</ns1:nomPatronymique><ns1:prenoms>Nadia</ns1:prenoms><ns1:nir>29510</ns1:nir><ns1:dateNaissance>1995-10-22</ns1:dateNaissance><ns1:nationalite code="1"/><ns1:telDomicilePro>0341589894</ns1:telDomicilePro><ns1:telPortable>0615898941</ns1:telPortable><ns1:mel>nadia.paris-leclerc@yopmail.com</ns1:mel><ns1:notifMel>false</ns1:notifMel><ns1:notifSms>false</ns1:notifSms><ns1:adressePostale><ns1:etranger>false</ns1:etranger><ns1:numero>15</ns1:numero><ns1:voie>Rue Saint-Michel</ns1:voie><ns1:lieuDit/><ns1:codePostal>75000</ns1:codePostal><ns1:commune code="75056"/></ns1:adressePostale><ns1:situationProfessionnelle><ns1:typeContratTravail code="ASS"/><ns1:etranger>false</ns1:etranger><ns1:codePostal>75000</ns1:codePostal><ns1:commune code="75056"/><ns1:siretEmployeur>55204944792805</ns1:siretEmployeur><ns1:cddInterim>false</ns1:cddInterim><ns1:periodeChomage>false</ns1:periodeChomage></ns1:situationProfessionnelle><ns1:revenuFiscal><ns1:anneeMoins2>2021</ns1:anneeMoins2><ns1:montantMoins2>5000</ns1:montantMoins2></ns1:revenuFiscal><ns1:listeRessourceRecue><ns1:detailRessource><ns1:ressource code="SAL"/><ns1:montant>2500</ns1:montant></ns1:detailRessource></ns1:listeRessourceRecue><ns1:listeRessourceVersee/><ns1:listeRessourceRecuePersACharge/><ns1:listeRessourceVerseePersACharge/></ns1:demandeur><ns1:adresseLogement><ns1:etranger>false</ns1:etranger><ns1:numero>15</ns1:numero><ns1:voie>Rue Saint-Michel</ns1:voie><ns1:lieuDit/><ns1:codePostal>75000</ns1:codePostal><ns1:commune code="75056"/></ns1:adresseLogement><ns1:listeCodemandeur/><ns1:situation><ns1:listeModeleLogement><ns1:modeleLogement code="LSLLP"/></ns1:listeModeleLogement><ns1:listeSituation><ns1:detailSituation code="TTC"/><ns1:detailSituation code="TTP"/></ns1:listeSituation><ns1:montantMensuelLoyer>0</ns1:montantMensuelLoyer><ns1:montantMensuelAPL>0</ns1:montantMensuelAPL><ns1:nombreHabitant>0</ns1:nombreHabitant><ns1:surface>0</ns1:surface><ns1:proprietaire>false</ns1:proprietaire></ns1:situation></ns1:personnePhysique><ns1:informationHandicap><ns1:nombreHandicap>0</ns1:nombreHandicap></ns1:informationHandicap></ns1:demandeLogement></ns1:demande></ns1:interfaceNuu>'
      )
    end
   
    describe 'using to_xml' do
      let!(:add_request_task1) do
        build(
            :add_request_task,
            options: {housing_request_id: hr.id }
        )
      end
      let!(:so) { so=Hermes::BoClient::AddRequestTask.new({:'housing_request_id'=> hr.id})    }

      it 'should return a valid xml' do
        add_request_task1.validate_xml
        expect(hr.reload.last_xml_validation_report).to eq('[]')

        VCR.use_cassette("flux30", :match_requests_on => [:method, :uri, :path]) do
          add_request_task1.execute
          last_job = enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.last
          last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        end

        expect(hr.reload.last_send_has_error).to eq(false)
        expect(Hermes::XmlSopraError.count).to eq(0)
      end

      it 'throw 404 error' do 
        VCR.use_cassette("flux30_not_found", :match_requests_on => [:method, :uri, :path]) do
          add_request_task1.execute
          last_job = enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.last
          last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        end

        expect(hr.reload.last_send_has_error).to eq(true)
        expect(Hermes::XmlSopraError.count).to eq(1)
        xml_error = Hermes::XmlSopraError.last
        expect(xml_error.code).to eq("404")
        expect(xml_error.error_content).to include("RestClient::NotFound ---->")
      end 

    end
  end
end